@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -58,9 +58,14 @@ typedef __m128i vint;
 typedef __m256 vfloat;
 typedef struct { __m128i x, y; } vint2;
 
+typedef __m256i vint64;
+typedef __m256i vuint64;
+
 typedef struct {
   vmask x, y;
-} vmask2;
+} vquad;
+
+typedef vquad vargquad;
 
 //
 
@@ -84,13 +89,10 @@ static INLINE int cpuSupportsFMA4() {
 
 #if CONFIG == 4 && defined(__AVX__) && defined(__FMA4__)
 static INLINE int vavailability_i(int name) {
-  //int d = __builtin_cpu_supports("avx") && __builtin_cpu_supports("fma4");
   int d = cpuSupportsAVX() && cpuSupportsFMA4();
   return d ? 3 : 0;
 }
 
-//typedef vint2 vint2_fma4;
-
 #define ENABLE_FMA_DP
 #define ENABLE_FMA_SP
 
@@ -101,7 +103,6 @@ static INLINE int vavailability_i(int name) {
   int d = cpuSupportsAVX();
   return d ? 3 : 0;
 }
-//typedef vint2 vint2_avx;
 
 #define ISANAME "AVX"
 #define DFTPRIORITY 20
@@ -124,17 +125,6 @@ static INLINE int vtestallones_i_vo64(vopmask g) {
 static INLINE vdouble vcast_vd_d(double d) { return _mm256_set1_pd(d); }
 static INLINE vmask vreinterpret_vm_vd(vdouble vd) { return _mm256_castpd_si256(vd); }
 static INLINE vdouble vreinterpret_vd_vm(vmask vm) { return _mm256_castsi256_pd(vm);  }
-static INLINE vint2 vreinterpret_vi2_vd(vdouble vd) {
-  vint2 r;
-  r.x = _mm256_castsi256_si128(vreinterpret_vm_vd(vd));
-  r.y = _mm256_extractf128_si256(vreinterpret_vm_vd(vd), 1);
-  return r;
-}
-static INLINE vdouble vreinterpret_vd_vi2(vint2 vi) {
-  vmask m = _mm256_castsi128_si256(vi.x);
-  m = _mm256_insertf128_si256(m, vi.y, 1);
-  return vreinterpret_vd_vm(m);
-}
 
 //
 
@@ -183,6 +173,8 @@ static INLINE vopmask vcast_vo64_vo32(vopmask o) {
   return vreinterpret_vm_vd(_mm256_cmp_pd(_mm256_cvtepi32_pd(_mm256_castsi256_si128(o)), _mm256_set1_pd(-1.0), _CMP_EQ_OQ));
 }
 
+static INLINE vopmask vcast_vo_i(int i) { return _mm256_set1_epi64x(i ? -1 : 0); }
+
 //
 
 static INLINE vint vrint_vi_vd(vdouble vd) { return _mm256_cvtpd_epi32(vd); }
@@ -193,16 +185,15 @@ static INLINE vfloat vrint_vf_vf(vfloat vd) { return _mm256_round_ps(vd, _MM_FRO
 static INLINE vfloat vtruncate_vf_vf(vfloat vf) { return _mm256_round_ps(vf, _MM_FROUND_TO_ZERO |_MM_FROUND_NO_EXC); }
 static INLINE vdouble vcast_vd_vi(vint vi) { return _mm256_cvtepi32_pd(vi); }
 static INLINE vint vcast_vi_i(int i) { return _mm_set1_epi32(i); }
-static INLINE vint2 vcastu_vi2_vi(vint vi) {
-  vint2 r;
-  r.x = _mm_and_si128(_mm_shuffle_epi32(vi, 0x40), _mm_set_epi32(-1, 0, -1, 0));
-  r.y = _mm_and_si128(_mm_shuffle_epi32(vi, 0xc8), _mm_set_epi32(-1, 0, -1, 0));
-  return r;
+
+static INLINE vmask vcastu_vm_vi(vint vi) {
+  __m256i m = _mm256_castsi128_si256(_mm_and_si128(_mm_shuffle_epi32(vi, 0x40), _mm_set_epi32(-1, 0, -1, 0)));
+  return _mm256_insertf128_si256(m,  _mm_and_si128(_mm_shuffle_epi32(vi, 0xc8), _mm_set_epi32(-1, 0, -1, 0)), 1);
 }
 
-static INLINE vint vcastu_vi_vi2(vint2 vi) {
-  return _mm_or_si128(_mm_and_si128(_mm_shuffle_epi32(vi.x, 0x0d), _mm_set_epi32( 0,  0, -1, -1)),
-		      _mm_and_si128(_mm_shuffle_epi32(vi.y, 0xd0), _mm_set_epi32(-1, -1,  0,  0)));
+static INLINE vint vcastu_vi_vm(vmask vi) {
+  return _mm_or_si128(_mm_and_si128(_mm_shuffle_epi32(_mm256_castsi256_si128(vi)     , 0x0d), _mm_set_epi32( 0,  0, -1, -1)),
+		      _mm_and_si128(_mm_shuffle_epi32(_mm256_extractf128_si256(vi, 1), 0xd0), _mm_set_epi32(-1, -1,  0,  0)));
 }
 
 static INLINE vmask vcast_vm_i_i(int i0, int i1) {
@@ -213,6 +204,9 @@ static INLINE vopmask veq64_vo_vm_vm(vmask x, vmask y) {
   return vreinterpret_vm_vd(_mm256_cmp_pd(vreinterpret_vd_vm(vxor_vm_vm_vm(vxor_vm_vm_vm(x, y), vreinterpret_vm_vd(_mm256_set1_pd(1.0)))), _mm256_set1_pd(1.0), _CMP_EQ_OQ));
 }
 
+static INLINE vmask vcast_vm_i64(int64_t i) { return _mm256_set1_epi64x(i); }
+static INLINE vmask vcast_vm_u64(uint64_t i) { return _mm256_set1_epi64x((uint64_t)i); }
+
 //
 
 static INLINE vdouble vadd_vd_vd_vd(vdouble x, vdouble y) { return _mm256_add_pd(x, y); }
@@ -562,7 +556,6 @@ static INLINE void vsscatter2_v_p_i_i_vd(double *ptr, int offset, int step, vdou
 
 static INLINE vfloat vrev21_vf_vf(vfloat d0) { return _mm256_shuffle_ps(d0, d0, (2 << 6) | (3 << 4) | (0 << 2) | (1 << 0)); }
 static INLINE vfloat vreva2_vf_vf(vfloat d0) { d0 = _mm256_permute2f128_ps(d0, d0, 1); return _mm256_shuffle_ps(d0, d0, (1 << 6) | (0 << 4) | (3 << 2) | (2 << 0)); }
-static INLINE vint2 vrev21_vi2_vi2(vint2 i) { return vreinterpret_vi2_vf(vrev21_vf_vf(vreinterpret_vf_vi2(i))); }
 
 static INLINE void vstream_v_p_vf(float *ptr, vfloat v) { _mm256_stream_ps(ptr, v); }
 
@@ -577,70 +570,23 @@ static INLINE void vsscatter2_v_p_i_i_vf(float *ptr, int offset, int step, vfloa
 
 //
 
-static INLINE vmask2 vinterleave_vm2_vm2(vmask2 v) {
-  return (vmask2) {
-    vreinterpret_vm_vd(_mm256_unpacklo_pd(vreinterpret_vd_vm(v.x), vreinterpret_vd_vm(v.y))),
-      vreinterpret_vm_vd(_mm256_unpackhi_pd(vreinterpret_vd_vm(v.x), vreinterpret_vd_vm(v.y))) };
+static vquad loadu_vq_p(void *p) {
+  vquad vq;
+  memcpy(&vq, p, VECTLENDP * 16);
+  return vq;
 }
 
-static INLINE vmask2 vuninterleave_vm2_vm2(vmask2 v) {
-  return (vmask2) {
-    vreinterpret_vm_vd(_mm256_unpacklo_pd(vreinterpret_vd_vm(v.x), vreinterpret_vd_vm(v.y))),
-      vreinterpret_vm_vd(_mm256_unpackhi_pd(vreinterpret_vd_vm(v.x), vreinterpret_vd_vm(v.y))) };
+static INLINE vquad cast_vq_aq(vargquad aq) {
+  vquad vq;
+  memcpy(&vq, &aq, VECTLENDP * 16);
+  return vq;
 }
 
-static INLINE vint vuninterleave_vi_vi(vint v) {
-  return _mm_shuffle_epi32(v, (0 << 0) | (2 << 2) | (1 << 4) | (3 << 6));
-}
-
-static INLINE vdouble vinterleave_vd_vd(vdouble vd) {
-  double tmp[4];
-  vstoreu_v_p_vd(tmp, vd);
-  double t = tmp[1]; tmp[1] = tmp[2]; tmp[2] = t;
-  return vloadu_vd_p(tmp);
-}
-
-static INLINE vdouble vuninterleave_vd_vd(vdouble vd) {
-  double tmp[4];
-  vstoreu_v_p_vd(tmp, vd);
-  double t = tmp[1]; tmp[1] = tmp[2]; tmp[2] = t;
-  return vloadu_vd_p(tmp);
-}
-
-static INLINE vmask vinterleave_vm_vm(vmask vm) {
-  double tmp[4];
-  vstoreu_v_p_vd(tmp, vreinterpret_vd_vm(vm));
-  double t = tmp[1]; tmp[1] = tmp[2]; tmp[2] = t;
-  return vreinterpret_vm_vd(vloadu_vd_p(tmp));
-}
-
-static INLINE vmask vuninterleave_vm_vm(vmask vm) {
-  double tmp[4];
-  vstoreu_v_p_vd(tmp, vreinterpret_vd_vm(vm));
-  double t = tmp[1]; tmp[1] = tmp[2]; tmp[2] = t;
-  return vreinterpret_vm_vd(vloadu_vd_p(tmp));
-}
-
-static vmask2 vloadu_vm2_p(void *p) {
-  vmask2 vm2;
-  memcpy(&vm2, p, VECTLENDP * 16);
-  return vm2;
-}
-
-#if !defined(SLEEF_GENHEADER)
-typedef Sleef_quad4 vargquad;
-
-static INLINE vmask2 vcast_vm2_aq(vargquad aq) {
-  return vinterleave_vm2_vm2(vloadu_vm2_p(&aq));
-}
-
-static INLINE vargquad vcast_aq_vm2(vmask2 vm2) {
-  vm2 = vuninterleave_vm2_vm2(vm2);
+static INLINE vargquad cast_aq_vq(vquad vq) {
   vargquad aq;
-  memcpy(&aq, &vm2, VECTLENDP * 16);
+  memcpy(&aq, &vq, VECTLENDP * 16);
   return aq;
 }
-#endif // #if !defined(SLEEF_GENHEADER)
 
 static INLINE int vtestallzeros_i_vo64(vopmask g) {
   return _mm_movemask_epi8(_mm_or_si128(_mm256_extractf128_si256(g, 0), _mm256_extractf128_si256(g, 1))) == 0;
@@ -679,9 +625,14 @@ static INLINE vmask vcast_vm_vi(vint vi) {
   vint vi0 = _mm_and_si128(_mm_shuffle_epi32(vi, (1 << 4) | (1 << 6)), _mm_set_epi32(0, -1, 0, -1));
   vint vi1 = _mm_and_si128(_mm_shuffle_epi32(vi, (2 << 0) | (2 << 2) | (3 << 4) | (3 << 6)), _mm_set_epi32(0, -1, 0, -1));
   vmask m = _mm256_insertf128_si256(_mm256_castsi128_si256(vi0), vi1, 1);
-  return vor_vm_vm_vm(vcast_vm_vi2(vcastu_vi2_vi(vand_vi_vo_vi(vgt_vo_vi_vi(vcast_vi_i(0), vi), vcast_vi_i(-1)))), m);
+  return vor_vm_vm_vm(vcastu_vm_vi(vand_vi_vo_vi(vgt_vo_vi_vi(vcast_vi_i(0), vi), vcast_vi_i(-1))), m);
 }
 static INLINE vint vcast_vi_vm(vmask vm) {
   return _mm_or_si128(_mm_castps_si128(_mm_shuffle_ps(_mm_castsi128_ps(_mm256_castsi256_si128(vm)), _mm_set1_ps(0), 0x08)),
   		      _mm_castps_si128(_mm_shuffle_ps(_mm_set1_ps(0), _mm_castsi128_ps(_mm256_extractf128_si256(vm, 1)), 0x80)));
 }
+
+static INLINE vmask vreinterpret_vm_vi64(vint64 v) { return v; }
+static INLINE vint64 vreinterpret_vi64_vm(vmask m) { return m; }
+static INLINE vmask vreinterpret_vm_vu64(vuint64 v) { return v; }
+static INLINE vuint64 vreinterpret_vu64_vm(vmask m) { return m; }
