@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2023.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -18,9 +18,9 @@
 #else
 #include <unistd.h>
 #include <sys/types.h>
-#include <signal.h>
 #endif
 
+#include "quaddef.h"
 #include "misc.h"
 
 #if !defined(USE_INLINE_HEADER)
@@ -57,6 +57,10 @@
 #include <arm_sve.h>
 #endif
 
+#if defined(__riscv) && defined(__riscv_v)
+#include <riscv_vector.h>
+#endif
+
 #if defined(__VSX__)
 #include <altivec.h>
 #endif
@@ -71,6 +75,10 @@
 #include USE_INLINE_HEADER
 #include MACRO_ONLY_HEADER
 
+#ifndef ENABLE_PUREC_SCALAR
+#include "sleefinline_purec_scalar.h"
+#endif
+
 #endif // #if !defined(USE_INLINE_HEADER)
 
 #include "testerutil.h"
@@ -220,8 +228,6 @@ typedef Sleef___m128_2 vfloat2;
 #if !defined(USE_INLINE_HEADER)
 #define CONFIG 1
 #include "helpersve.h"
-typedef Sleef_svfloat64_t_2 vdouble2;
-typedef Sleef_svfloat32_t_2 vfloat2;
 #endif
 #endif
 
@@ -230,8 +236,6 @@ typedef Sleef_svfloat32_t_2 vfloat2;
 #if !defined(USE_INLINE_HEADER)
 #define CONFIG 2
 #include "helpersve.h"
-typedef Sleef_svfloat64_t_2 vdouble2;
-typedef Sleef_svfloat32_t_2 vfloat2;
 #endif
 #endif
 
@@ -249,8 +253,8 @@ typedef Sleef___m256_2 vfloat2;
 #define CONFIG 1
 #include "helperpower_128.h"
 #include "renamevsx.h"
-typedef Sleef___vector_double_2 vdouble2;
-typedef Sleef___vector_float_2 vfloat2;
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 #endif
 
@@ -260,13 +264,35 @@ typedef Sleef___vector_float_2 vfloat2;
 #define CONFIG 2
 #include "helperpower_128.h"
 #include "renamevsxnofma.h"
-typedef Sleef___vector_double_2 vdouble2;
-typedef Sleef___vector_float_2 vfloat2;
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 #endif
 
-#ifdef ENABLE_ZVECTOR2
-#include "renamezvector2.h"
+#ifdef ENABLE_VSX3
+#include "renamevsx3.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 3
+#include "helperpower_128.h"
+#include "renamevsx3.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+#endif
+
+#ifdef ENABLE_VSX3NOFMA
+#include "renamevsx3nofma.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 4
+#include "helperpower_128.h"
+#include "renamevsx3nofma.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+#endif
+
+#ifdef ENABLE_VXE
+#include "renamevxe.h"
 #if !defined(USE_INLINE_HEADER)
 #define CONFIG 140
 #include "helpers390x_128.h"
@@ -275,8 +301,8 @@ typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 #endif
 
-#ifdef ENABLE_ZVECTOR2NOFMA
-#include "renamezvector2nofma.h"
+#ifdef ENABLE_VXENOFMA
+#include "renamevxenofma.h"
 #if !defined(USE_INLINE_HEADER)
 #define CONFIG 141
 #include "helpers390x_128.h"
@@ -285,6 +311,74 @@ typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 #endif
 
+#ifdef ENABLE_VXE2
+#include "renamevxe2.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 150
+#include "helpers390x_128.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+#endif
+
+#ifdef ENABLE_VXE2NOFMA
+#include "renamevxe2nofma.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 151
+#include "helpers390x_128.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+#endif
+
+#ifdef ENABLE_DSPPOWER_128
+#define CONFIG 1
+#include "helperpower_128.h"
+#include "renamedsp128.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_DSPS390X_128
+#define CONFIG 140
+#include "helpers390x_128.h"
+#include "renamedsp128.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_RVVM1
+#include "renamervvm1.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 1
+#include "helperrvv.h"
+#endif
+#endif
+
+#ifdef ENABLE_RVVM1NOFMA
+#include "renamervvm1nofma.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 2
+#include "helperrvv.h"
+#endif
+#endif
+
+#ifdef ENABLE_RVVM2
+#include "renamervvm2.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 1
+#include "helperrvv.h"
+#endif
+#endif
+
+#ifdef ENABLE_RVVM2NOFMA
+#include "renamervvm2nofma.h"
+#if !defined(USE_INLINE_HEADER)
+#define CONFIG 2
+#include "helperrvv.h"
+#endif
+#endif
+
 #ifdef ENABLE_PUREC_SCALAR
 #include "renamepurec_scalar.h"
 #if !defined(USE_INLINE_HEADER)
@@ -305,6 +399,14 @@ typedef Sleef_float_2 vfloat2;
 #endif
 #endif
 
+#ifdef ENABLE_DSP_SCALAR
+#include "renamedspscalar.h"
+#define CONFIG 1
+#include "helperpurec_scalar.h"
+typedef Sleef_double_2 vdouble2;
+typedef Sleef_float_2 vfloat2;
+#endif
+
 #ifdef USE_INLINE_HEADER
 #define CONCAT_SIMD_SUFFIX_(keyword, suffix) keyword ## suffix
 #define CONCAT_SIMD_SUFFIX(keyword, suffix) CONCAT_SIMD_SUFFIX_(keyword, suffix)
@@ -330,47 +432,45 @@ typedef Sleef_float_2 vfloat2;
 
 //
 
+int check_feature(double d, float f) {
 #ifdef ENABLE_DP
-int check_featureDP(double d) {
-  double s[VECTLENDP];
-  int i;
-  for(i=0;i<VECTLENDP;i++) {
-    s[i] = d;
+  {
+    double s[VECTLENDP];
+    int i;
+    for(i=0;i<VECTLENDP;i++) {
+      s[i] = d;
+    }
+    vdouble a = vloadu_vd_p(s);
+    a = xpow(a, a);
+    vstoreu_v_p_vd(s, a);
+    if (s[0] == s[0]) return 1;
   }
-  vdouble a = vloadu_vd_p(s);
-  a = xpow(a, a);
-  vstoreu_v_p_vd(s, a);
-  return s[0] == s[0];
+#endif
+#ifdef ENABLE_SP
+  {
+    float s[VECTLENSP];
+    int i;
+    for(i=0;i<VECTLENSP;i++) {
+      s[i] = d;
+    }
+    vfloat a = vloadu_vf_p(s);
+    a = xpowf(a, a);
+    vstoreu_v_p_vf(s, a);
+    if (s[0] == s[0]) return 1;
+  }
+#endif
+  return 0;
 }
 
-#if !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA) || defined(USE_INLINE_HEADER))
+#if defined(ENABLE_DP) && !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA) || defined(ENABLE_RVVM1) || defined(ENABLE_RVVM1NOFMA) || defined(ENABLE_RVVM2) || defined(ENABLE_RVVM2NOFMA) || defined(USE_INLINE_HEADER))
 static vdouble vd2getx_vd_vd2(vdouble2 v) { return v.x; }
 static vdouble vd2gety_vd_vd2(vdouble2 v) { return v.y; }
 #endif
-#else // #ifdef ENABLE_DP
-int check_featureDP() { return 0; }
-#endif // #ifdef ENABLE_DP
 
-#ifdef ENABLE_SP
-int check_featureSP(float d) {
-  float s[VECTLENSP];
-  int i;
-  for(i=0;i<VECTLENSP;i++) {
-    s[i] = d;
-  }
-  vfloat a = vloadu_vf_p(s);
-  a = xpowf(a, a);
-  vstoreu_v_p_vf(s, a);
-  return s[0] == s[0];
-}
-
-#if !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA) || defined(USE_INLINE_HEADER))
+#if defined(ENABLE_SP) && !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA) || defined(ENABLE_RVVM1) || defined(ENABLE_RVVM1NOFMA) || defined(ENABLE_RVVM2) || defined(ENABLE_RVVM2NOFMA) || defined(USE_INLINE_HEADER))
 static vfloat vf2getx_vf_vf2(vfloat2 v) { return v.x; }
 static vfloat vf2gety_vf_vf2(vfloat2 v) { return v.y; }
 #endif
-#else // #ifdef ENABLE_DP
-int check_featureSP() { return 0; }
-#endif
 
 //
 
@@ -546,7 +646,7 @@ int check_featureSP() { return 0; }
 
 #define BUFSIZE 1024
 
-int do_test(int argc, char **argv) {
+int main2(int argc, char **argv) {
   xsrand(time(NULL));
 
   {
