@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2023.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -20,6 +20,7 @@
 #endif
 
 #include "sleef.h"
+#include "quaddef.h"
 #include "testerutil.h"
 
 #ifdef ENABLE_SSE2
@@ -130,34 +131,98 @@ typedef Sleef_float32x4_t_2 vfloat2;
 #define CONFIG 1
 #include "helperpower_128.h"
 #include "renamevsx.h"
-typedef Sleef___vector_double_2 vdouble2;
-typedef Sleef___vector_float_2 vfloat2;
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
 #ifdef ENABLE_VSXNOFMA
 #define CONFIG 2
 #include "helperpower_128.h"
 #include "renamevsxnofma.h"
-typedef Sleef___vector_double_2 vdouble2;
-typedef Sleef___vector_float_2 vfloat2;
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
-#ifdef ENABLE_ZVECTOR2
+#ifdef ENABLE_VSX3
+#define CONFIG 3
+#include "helperpower_128.h"
+#include "renamevsx3.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_VSX3NOFMA
+#define CONFIG 4
+#include "helperpower_128.h"
+#include "renamevsx3nofma.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_VXE
 #define CONFIG 140
 #include "helpers390x_128.h"
-#include "renamezvector2.h"
+#include "renamevxe.h"
 typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
 typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
-#ifdef ENABLE_ZVECTOR2NOFMA
+#ifdef ENABLE_VXENOFMA
 #define CONFIG 141
 #include "helpers390x_128.h"
-#include "renamezvector2nofma.h"
+#include "renamevxenofma.h"
 typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
 typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
+#ifdef ENABLE_VXE2
+#define CONFIG 150
+#include "helpers390x_128.h"
+#include "renamevxe2.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_VXE2NOFMA
+#define CONFIG 151
+#include "helpers390x_128.h"
+#include "renamevxe2nofma.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_RVVM1
+#define CONFIG 1
+#define ENABLE_RVV_DP
+#include "helperrvv.h"
+#include "renamervvm1.h"
+#include "sleef.h"
+#endif
+
+#ifdef ENABLE_RVVM1NOFMA
+#define CONFIG 2
+#define ENABLE_RVV_DP
+#include "helperrvv.h"
+#include "renamervvm1nofma.h"
+#include "sleef.h"
+#endif
+
+#ifdef ENABLE_RVVM2
+#define CONFIG 1
+#define ENABLE_RVV_DP
+#include "helperrvv.h"
+#include "renamervvm2.h"
+#include "sleef.h"
+#endif
+
+#ifdef ENABLE_RVVM2NOFMA
+#define CONFIG 2
+#define ENABLE_RVV_DP
+#include "helperrvv.h"
+#include "renamervvm2nofma.h"
+#include "sleef.h"
+#endif
+
 #ifdef ENABLE_PUREC_SCALAR
 #define CONFIG 1
 #include "helperpurec_scalar.h"
@@ -176,7 +241,7 @@ typedef Sleef_float_2 vfloat2;
 
 //
 
-#if !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA))
+#if !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA) || defined(ENABLE_RVVM1) || defined(ENABLE_RVVM1NOFMA) || defined(ENABLE_RVVM2) || defined(ENABLE_RVVM2NOFMA))
 static vdouble vd2getx_vd_vd2(vdouble2 v) { return v.x; }
 static vdouble vd2gety_vd_vd2(vdouble2 v) { return v.y; }
 #endif
@@ -1051,7 +1116,6 @@ int main(int argc,char **argv)
       mpfr_hypot(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULP2dp(t = vget(xhypot_u05(vd, vd2), e), frx);
-      double c = mpfr_get_d(frx, GMP_RNDN);
 
       if (u0 > 0.5) {
 	printf(ISANAME " hypot_u05 arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -1107,7 +1171,6 @@ int main(int argc,char **argv)
       mpfr_fmod(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULPdp(t = vget(xfmod(vd, vd2), e), frx);
-      long double c = mpfr_get_ld(frx, GMP_RNDN);
 
       if (fabsl((long double)d / d2) < 1e+300 && u0 > 0.5) {
 	printf(ISANAME " fmod arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -1122,7 +1185,6 @@ int main(int argc,char **argv)
       mpfr_remainder(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULPdp(t = vget(xremainder(vd, vd2), e), frx);
-      long double c = mpfr_get_ld(frx, GMP_RNDN);
 
       if (fabsl((long double)d / d2) < 1e+300 && u0 > 0.5) {
 	printf(ISANAME " remainder arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -1183,7 +1245,7 @@ int main(int argc,char **argv)
       double u0 = countULPdp(t = vget(xlgamma_u1(vd), e), frx);
 
       if (((d < 0 && fabsl(t - mpfr_get_ld(frx, GMP_RNDN)) > 1e-15 && u0 > 1) || (0 <= d && d < 2e+305 && u0 > 1) || (2e+305 <= d && !(u0 <= 1 || isinf(t))))) {
-	printf("Pure C xlgamma_u1 arg=%.20g ulp=%.20g\n", d, u0);
+	printf(ISANAME " xlgamma_u1 arg=%.20g ulp=%.20g\n", d, u0);
 	printf("Correct = %.20Lg, test = %.20g\n", mpfr_get_ld(frx, GMP_RNDN), t);
 	printf("Diff = %.20Lg\n", fabsl(t - mpfr_get_ld(frx, GMP_RNDN)));
 	fflush(stdout); ecnt++;
@@ -1197,7 +1259,7 @@ int main(int argc,char **argv)
       double u0 = countULP2dp(t = vget(xtgamma_u1(vd), e), frx);
 
       if (u0 > 1.0) {
-	printf("Pure C xtgamma_u1 arg=%.20g ulp=%.20g\n", d, u0);
+	printf(ISANAME " xtgamma_u1 arg=%.20g ulp=%.20g\n", d, u0);
 	printf("Correct = %.20g, test = %.20g\n", mpfr_get_d(frx, GMP_RNDN), t);
 	printf("Diff = %.20Lg\n", fabsl(t - mpfr_get_ld(frx, GMP_RNDN)));
 	fflush(stdout); ecnt++;
@@ -1213,7 +1275,7 @@ int main(int argc,char **argv)
       double u0 = countULP2dp(t = vget(xerfc_u15(vd), e), frx);
 
       if ((d > 26.2 && u0 > 2.5 && !(mpfr_get_d(frx, GMP_RNDN) == 0 && t <= ebz)) || (d <= 26.2 && u0 > 1.5)) {
-	printf("Pure C xerfc_u15 arg=%.20g ulp=%.20g\n", d, u0);
+	printf(ISANAME " xerfc_u15 arg=%.20g ulp=%.20g\n", d, u0);
 	printf("Correct = %.20Lg, test = %.20g\n", mpfr_get_ld(frx, GMP_RNDN), t);
 	fflush(stdout); ecnt++;
       }
@@ -1225,11 +1287,12 @@ int main(int argc,char **argv)
 
       double u0 = countULP2dp(t = vget(xerf_u1(vd), e), frx);
 
-      if (u0 > 1) {
-	printf("Pure C xerf_u1 arg=%.20g ulp=%.20g\n", d, u0);
+      if (u0 > 0.75) {
+	printf(ISANAME " xerf_u1 arg=%.20g ulp=%.20g\n", d, u0);
 	printf("Correct = %.20Lg, test = %.20g\n", mpfr_get_ld(frx, GMP_RNDN), t);
 	fflush(stdout); ecnt++;
       }
     }
   }
+  mpfr_clears(frw, frx, fry, frz, NULL);
 }
