@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -26,8 +26,10 @@ extern const double Sleef_rempitabdp[];
 #include "rename.h"
 #endif
 
-#if (defined(_MSC_VER))
+#if defined(_MSC_VER) && !defined (__clang__)
 #pragma fp_contract (off)
+#else
+#pragma STDC FP_CONTRACT OFF
 #endif
 
 #define MLA mla
@@ -35,21 +37,15 @@ extern const double Sleef_rempitabdp[];
 #include "estrin.h"
 
 static INLINE CONST int64_t doubleToRawLongBits(double d) {
-  union {
-    double f;
-    int64_t i;
-  } tmp;
-  tmp.f = d;
-  return tmp.i;
+  int64_t ret;
+  memcpy(&ret, &d, sizeof(ret));
+  return ret;
 }
 
 static INLINE CONST double longBitsToDouble(int64_t i) {
-  union {
-    double f;
-    int64_t i;
-  } tmp;
-  tmp.i = i;
-  return tmp.f;
+  double ret;
+  memcpy(&ret, &i, sizeof(ret));
+  return ret;
 }
 
 static INLINE CONST double fabsk(double x) {
@@ -72,6 +68,7 @@ static INLINE CONST double trunck(double x) { return (double)(int)x; }
 static INLINE CONST double fmink(double x, double y) { return x < y ? x : y; }
 static INLINE CONST double fmaxk(double x, double y) { return x > y ? x : y; }
 
+static INLINE CONST int xsignbit(double d) { return (doubleToRawLongBits(d) & doubleToRawLongBits(-0.0)) == doubleToRawLongBits(-0.0); }
 static INLINE CONST int xisnan(double x) { return x != x; }
 static INLINE CONST int xisinf(double x) { return x == SLEEF_INFINITY || x == -SLEEF_INFINITY; }
 static INLINE CONST int xisminf(double x) { return x == -SLEEF_INFINITY; }
@@ -623,7 +620,7 @@ static Sleef_double2 atan2k_u1(Sleef_double2 y, Sleef_double2 x) {
   t = ddsqu_d2_d2(s);
   t = ddnormalize_d2_d2(t);
 
-  double t2 = t.x * t.x, t4 = t2 * t2, t8 = t4 * t4, t16 = t8 * t8;
+  double t2 = t.x * t.x, t4 = t2 * t2, t8 = t4 * t4;
   u = POLY16(t.x, t2, t4, t8,
 	     1.06298484191448746607415e-05,
 	     -0.000125620649967286867384336,
@@ -700,7 +697,7 @@ EXPORT CONST double xasin_u1(double d) {
 EXPORT CONST double xacos_u1(double d) {
   int o = fabsk(d) < 0.5;
   double x2 = o ? (d*d) : ((1-fabsk(d))*0.5), u;
-  Sleef_double2 x = o ? dd(fabsk(d), 0) : ddsqrt_d2_d(x2), w;
+  Sleef_double2 x = o ? dd(fabsk(d), 0) : ddsqrt_d2_d(x2);
   x = fabsk(d) == 1.0 ? dd(0, 0) : x;
 
   double x4 = x2 * x2, x8 = x4 * x4, x16 = x8 * x8;
@@ -764,9 +761,8 @@ static CONST di_t rempisub(double x) {
 
 // Payne-Hanek like argument reduction
 static CONST ddi_t rempi(double a) {
-  Sleef_double2 x, y, z;
+  Sleef_double2 x, y;
   di_t di;
-  double t;
   int ex = ilogb2k(a) - 55, q = ex > (700-55) ? -64 : 0;
   a = ldexp3k(a, q);
   if (ex < 0) ex = 0;
@@ -1644,14 +1640,13 @@ EXPORT CONST double xpow(double x, double y) {
 
   Sleef_double2 d = ddmul_d2_d2_d(logk(fabsk(x)), y);
   double result = expk(d);
-  if (d.x > 709.78271114955742909217217426) result = SLEEF_INFINITY;
 
-  result = xisnan(result) ? SLEEF_INFINITY : result;
-  result *= (x > 0 ? 1 : (!yisint ? SLEEF_NAN : (yisodd ? -1 : 1)));
+  result = (d.x > 709.78271114955742909217217426 || xisnan(result)) ? SLEEF_INFINITY : result;
+  result *= (x > 0 ? 1 : (yisint ? (yisodd ? -1 : 1) : SLEEF_NAN));
 
   double efx = mulsign(fabsk(x) - 1, y);
   if (xisinf(y)) result = efx < 0 ? 0.0 : (efx == 0 ? 1.0 : SLEEF_INFINITY);
-  if (xisinf(x) || x == 0) result = (yisodd ? sign(x) : 1) * ((x == 0 ? -y : y) < 0 ? 0 : SLEEF_INFINITY);
+  if (xisinf(x) || x == 0) result = mulsign((xsignbit(y) ^ (x == 0)) ? 0 : SLEEF_INFINITY, yisodd ? x : 1);
   if (xisnan(x) || xisnan(y)) result = SLEEF_NAN;
   if (y == 0 || x == 1) result = 1;
 
@@ -2334,57 +2329,59 @@ EXPORT CONST double xhypot_u35(double x, double y) {
 }
 
 EXPORT CONST double xnextafter(double x, double y) {
-  union {
-    double f;
-    int64_t i;
-  } cx;
+  double cxf;
+  int64_t cxi;
 
   x = x == 0 ? mulsign(0, y) : x;
-  cx.f = x;
-  int c = (cx.i < 0) == (y < x);
-  if (c) cx.i = -(cx.i ^ (UINT64_C(1) << 63));
+  cxf = x;
+  memcpy(&cxi, &cxf, sizeof(cxi));
 
-  if (x != y) cx.i--;
+  int c = (cxi < 0) == (y < x);
+  if (c) cxi = -(cxi ^ (int64_t)(UINT64_C(1) << 63));
 
-  if (c) cx.i = -(cx.i ^ (UINT64_C(1) << 63));
+  if (x != y) cxi--;
 
-  if (cx.f == 0 && x != 0) cx.f = mulsign(0, x);
-  if (x == 0 && y == 0) cx.f = y;
-  if (xisnan(x) || xisnan(y)) cx.f = SLEEF_NAN;
+  if (c) cxi = -(cxi ^ (int64_t)(UINT64_C(1) << 63));
+
+  memcpy(&cxf, &cxi, sizeof(cxf));
+  if (cxf == 0 && x != 0) cxf = mulsign(0, x);
+  if (x == 0 && y == 0) cxf = y;
+  if (xisnan(x) || xisnan(y)) cxf = SLEEF_NAN;
   
-  return cx.f;
+  return cxf;
 }
 
 EXPORT CONST double xfrfrexp(double x) {
-  union {
-    double f;
-    uint64_t u;
-  } cx;
+  double cxf;
+  uint64_t cxu;
 
   if (fabsk(x) < DBL_MIN) x *= (UINT64_C(1) << 63);
   
-  cx.f = x;
-  cx.u &= ~UINT64_C(0x7ff0000000000000);
-  cx.u |=  UINT64_C(0x3fe0000000000000);
+  cxf = x;
+  memcpy(&cxu, &cxf, sizeof(cxu));
 
-  if (xisinf(x)) cx.f = mulsign(SLEEF_INFINITY, x);
-  if (x == 0) cx.f = x;
+  cxu &= ~UINT64_C(0x7ff0000000000000);
+  cxu |=  UINT64_C(0x3fe0000000000000);
+
+  memcpy(&cxf, &cxu, sizeof(cxf));
+  if (xisinf(x)) cxf = mulsign(SLEEF_INFINITY, x);
+  if (x == 0) cxf = x;
   
-  return cx.f;
+  return cxf;
 }
 
 EXPORT CONST int xexpfrexp(double x) {
-  union {
-    double f;
-    uint64_t u;
-  } cx;
+  double cxf;
+  uint64_t cxu;
 
   int ret = 0;
   
   if (fabsk(x) < DBL_MIN) { x *= (UINT64_C(1) << 63); ret = -63; }
   
-  cx.f = x;
-  ret += (int32_t)(((cx.u >> 52) & 0x7ff)) - 0x3fe;
+  cxf = x;
+  memcpy(&cxu, &cxf, sizeof(cxu));
+
+  ret += (int32_t)(((cxu >> 52) & 0x7ff)) - 0x3fe;
 
   if (x == 0 || xisnan(x) || xisinf(x)) ret = 0;
   
@@ -2471,7 +2468,7 @@ typedef struct {
 } dd2;
 
 static CONST dd2 gammak(double a) {
-  Sleef_double2 clc = dd(0, 0), clln = dd(1, 0), clld = dd(1, 0), v = dd(1, 0), x, y, z;
+  Sleef_double2 clc = dd(0, 0), clln = dd(1, 0), clld = dd(1, 0), x, y, z;
   double t, u;
 
   int otiny = fabsk(a) < 1e-306, oref = a < 0.5;
@@ -2559,42 +2556,90 @@ EXPORT CONST double xlgamma_u1(double a) {
   return r;
 }
 
-EXPORT CONST double xerf_u1(double a) {
-  double s = a, t, u;
-  Sleef_double2 d;
+static INLINE CONST Sleef_double2 ddmla(double x, Sleef_double2 y, Sleef_double2 z) {
+  return ddadd2_d2_d2_d2(z, ddmul_d2_d2_d(y, x));
+}
+static INLINE CONST Sleef_double2 poly2dd_b(double x, Sleef_double2 c1, Sleef_double2 c0) { return ddmla(x, c1, c0); }
+static INLINE CONST Sleef_double2 poly2dd(double x, double c1, Sleef_double2 c0) { return ddmla(x, dd(c1, 0), c0); }
+static INLINE CONST Sleef_double2 poly4dd(double x, double c3, Sleef_double2 c2, Sleef_double2 c1, Sleef_double2 c0) {
+  return ddmla(x*x, poly2dd(x, c3, c2), poly2dd_b(x, c1, c0));
+}
 
-  a = fabsk(a);
-  int o0 = a < 1.0, o1 = a < 3.7, o2 = a < 6.0;
-  u = o0 ? (a*a) : a;
-  
-  t = o0 ? +0.6801072401395392157e-20 : o1 ? +0.2830954522087717660e-13 : -0.5846750404269610493e-17;
-  t = mla(t, u, o0 ? -0.2161766247570056391e-18 : o1 ? -0.1509491946179481940e-11 : +0.6076691048812607898e-15);
-  t = mla(t, u, o0 ? +0.4695919173301598752e-17 : o1 ? +0.3827857177807173152e-10 : -0.3007518609604893831e-13);
-  t = mla(t, u, o0 ? -0.9049140419888010819e-16 : o1 ? -0.6139733921558987241e-09 : +0.9427906260824646063e-12);
-  t = mla(t, u, o0 ? +0.1634018903557411517e-14 : o1 ? +0.6985387934608038824e-08 : -0.2100110908269393629e-10);
-  t = mla(t, u, o0 ? -0.2783485786333455216e-13 : o1 ? -0.5988224513034371474e-07 : +0.3534639523461223473e-09);
-  t = mla(t, u, o0 ? +0.4463221276786412722e-12 : o1 ? +0.4005716952355346640e-06 : -0.4664967728285395926e-08);
-  t = mla(t, u, o0 ? -0.6711366622850138987e-11 : o1 ? -0.2132190104575784400e-05 : +0.4943823283769000532e-07);
-  t = mla(t, u, o0 ? +0.9422759050232658346e-10 : o1 ? +0.9092461304042630325e-05 : -0.4271203394761148254e-06);
-  t = mla(t, u, o0 ? -0.1229055530100228477e-08 : o1 ? -0.3079188080966205457e-04 : +0.3034067677404915895e-05);
-  t = mla(t, u, o0 ? +0.1480719281585085023e-07 : o1 ? +0.7971413443082370762e-04 : -0.1776295289066871135e-04);
-  t = mla(t, u, o0 ? -0.1636584469123402714e-06 : o1 ? -0.1387853215225442864e-03 : +0.8524547630559505050e-04);
-  t = mla(t, u, o0 ? +0.1646211436588923363e-05 : o1 ? +0.6469678026257590965e-04 : -0.3290582944961784398e-03);
-  t = mla(t, u, o0 ? -0.1492565035840624866e-04 : o1 ? +0.4996645280372945860e-03 : +0.9696966068789101157e-03);
-  t = mla(t, u, o0 ? +0.1205533298178966496e-03 : o1 ? -0.1622802482842520535e-02 : -0.1812527628046986137e-02);
-  t = mla(t, u, o0 ? -0.8548327023450851166e-03 : o1 ? +0.1615320557049377171e-03 : -0.4725409828123619017e-03);
-  t = mla(t, u, o0 ? +0.5223977625442188799e-02 : o1 ? +0.1915262325574875607e-01 : +0.2090315427924229266e-01);
-  t = mla(t, u, o0 ? -0.2686617064513125569e-01 : o1 ? -0.1027818298486033455e+00 : -0.1052041921842776645e+00);
-  t = mla(t, u, o0 ? +0.1128379167095512753e+00 : o1 ? -0.6366172819842503827e+00 : -0.6345351808766568347e+00);
-  t = mla(t, u, o0 ? -0.3761263890318375380e+00 : o1 ? -0.1128379590648910469e+01 : -0.1129442929103524396e+01);
-  d = ddmul_d2_d_d(t, u);
-  d = ddadd2_d2_d2_d2(d, o0 ? dd(1.1283791670955125586, 1.5335459613165822674e-17) :
-		      o1 ? dd(3.4110644736196137587e-08, -2.4875650708323294246e-24) :
-		      dd(0.00024963035690526438285, -5.4362665034856259795e-21));
-  d = o0 ? ddmul_d2_d2_d(d, a) : ddadd_d2_d_d2(1.0, ddneg_d2_d2(expk2(d)));
-  u = mulsign(o2 ? (d.x + d.y) : 1, s);
-  u = xisnan(a) ? SLEEF_NAN : u;
-  return u;
+EXPORT CONST double xerf_u1(double a) {
+  double t, x = fabsk(a);
+  Sleef_double2 t2;
+  double x2 = x * x, x4 = x2 * x2, x8 = x4 * x4, x16 = x8 * x8;
+
+  if (x < 2.5) {
+    // Abramowitz and Stegun
+    t = POLY21(x, x2, x4, x8, x16,
+	       -0.2083271002525222097e-14,
+	       +0.7151909970790897009e-13,
+	       -0.1162238220110999364e-11,
+	       +0.1186474230821585259e-10,
+	       -0.8499973178354613440e-10,
+	       +0.4507647462598841629e-9,
+	       -0.1808044474288848915e-8,
+	       +0.5435081826716212389e-8,
+	       -0.1143939895758628484e-7,
+	       +0.1215442362680889243e-7,
+	       +0.1669878756181250355e-7,
+	       -0.9808074602255194288e-7,
+	       +0.1389000557865837204e-6,
+	       +0.2945514529987331866e-6,
+	       -0.1842918273003998283e-5,
+	       +0.3417987836115362136e-5,
+	       +0.3860236356493129101e-5,
+	       -0.3309403072749947546e-4,
+	       +0.1060862922597579532e-3,
+	       +0.2323253155213076174e-3,
+	       +0.1490149719145544729e-3);
+    t2 = poly4dd(x, t,
+		 dd(0.0092877958392275604405, 7.9287559463961107493e-19),
+		 dd(0.042275531758784692937, 1.3785226620501016138e-19),
+		 dd(0.07052369794346953491, 9.5846628070792092842e-19));
+    t2 = ddadd_d2_d_d2(1, ddmul_d2_d2_d(t2, x));
+    t2 = ddsqu_d2_d2(t2);
+    t2 = ddsqu_d2_d2(t2);
+    t2 = ddsqu_d2_d2(t2);
+    t2 = ddsqu_d2_d2(t2);
+    t2 = ddrec_d2_d2(t2);
+  } else if (x > 6.0) {
+    t2 = dd(0, 0);
+  } else {
+    t = POLY21(x, x2, x4, x8, x16,
+	       -0.4024015130752621932e-18,
+	       +0.3847193332817048172e-16,
+	       -0.1749316241455644088e-14,
+	       +0.5029618322872872715e-13,
+	       -0.1025221466851463164e-11,
+	       +0.1573695559331945583e-10,
+	       -0.1884658558040203709e-9,
+	       +0.1798167853032159309e-8,
+	       -0.1380745342355033142e-7,
+	       +0.8525705726469103499e-7,
+	       -0.4160448058101303405e-6,
+	       +0.1517272660008588485e-5,
+	       -0.3341634127317201697e-5,
+	       -0.2515023395879724513e-5,
+	       +0.6539731269664907554e-4,
+	       -0.3551065097428388658e-3,
+	       +0.1210736097958368864e-2,
+	       -0.2605566912579998680e-2,
+	       +0.1252823202436093193e-2,
+	       +0.1820191395263313222e-1,
+	       -0.1021557155453465954e+0);
+    t2 = poly4dd(x, t,
+		 dd(-0.63691044383641748361, -2.4249477526539431839e-17),
+		 dd(-1.1282926061803961737, -6.2970338860410996505e-17),
+		 dd(-1.2261313785184804967e-05, -5.5329707514490107044e-22));
+    t2 = dd(expk(t2), 0);
+  }
+
+  t2 = ddadd2_d2_d2_d(t2, -1);
+
+  if (x < 1e-8) t2 = dd(-1.12837916709551262756245475959 * x, 0);
+  return mulsign(a == 0 ? 0 : (xisinf(a) ? 1 : (-t2.x - t2.y)), a);
 }
 
 EXPORT CONST double xerfc_u15(double a) {
