@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -26,8 +26,10 @@ extern const float Sleef_rempitabsp[];
 #include "rename.h"
 #endif
 
-#if (defined(_MSC_VER))
+#if defined(_MSC_VER) && !defined (__clang__)
 #pragma fp_contract (off)
+#else
+#pragma STDC FP_CONTRACT OFF
 #endif
 
 #define MLA mlaf
@@ -35,21 +37,15 @@ extern const float Sleef_rempitabsp[];
 #include "estrin.h"
 
 static INLINE CONST int32_t floatToRawIntBits(float d) {
-  union {
-    float f;
-    int32_t i;
-  } tmp;
-  tmp.f = d;
-  return tmp.i;
+  int32_t ret;
+  memcpy(&ret, &d, sizeof(ret));
+  return ret;
 }
 
 static INLINE CONST float intBitsToFloat(int32_t i) {
-  union {
-    float f;
-    int32_t i;
-  } tmp;
-  tmp.i = i;
-  return tmp.f;
+  float ret;
+  memcpy(&ret, &i, sizeof(ret));
+  return ret;
 }
 
 static INLINE CONST float fabsfk(float x) {
@@ -72,6 +68,7 @@ static INLINE CONST float fminfk(float x, float y) { return x < y ? x : y; }
 static INLINE CONST float fmaxfk(float x, float y) { return x > y ? x : y; }
 static INLINE CONST int xisintf(float x) { return (x == (int)x); }
 
+static INLINE CONST int xsignbitf(double d) { return (floatToRawIntBits(d) & floatToRawIntBits(-0.0)) == floatToRawIntBits(-0.0); }
 static INLINE CONST int xisnanf(float x) { return x != x; }
 static INLINE CONST int xisinff(float x) { return x == SLEEF_INFINITYf || x == -SLEEF_INFINITYf; }
 static INLINE CONST int xisminff(float x) { return x == -SLEEF_INFINITYf; }
@@ -439,9 +436,8 @@ static CONST fi_t rempisubf(float x) {
 }
 
 static CONST dfi_t rempif(float a) {
-  Sleef_float2 x, y, z;
+  Sleef_float2 x, y;
   fi_t di;
-  float t;
   int ex = ilogb2kf(a) - 25, q = ex > (90 - 25) ? -64 : 0;
   a = ldexp3kf(a, q);
   if (ex < 0) ex = 0;
@@ -1324,11 +1320,11 @@ EXPORT CONST float xpowf(float x, float y) {
   float result = expkf(dfmul_f2_f2_f(logkf(fabsfk(x)), y));
 
   result = xisnanf(result) ? SLEEF_INFINITYf : result;
-  result *=  (x >= 0 ? 1 : (!yisint ? SLEEF_NANf : (yisodd ? -1 : 1)));
+  result *= (x >= 0 ? 1 : (yisint ? (yisodd ? -1 : 1) : SLEEF_NANf));
 
   float efx = mulsignf(fabsfk(x) - 1, y);
   if (xisinff(y)) result = efx < 0 ? 0.0f : (efx == 0 ? 1.0f : SLEEF_INFINITYf);
-  if (xisinff(x) || x == 0) result = (yisodd ? signf(x) : 1) * ((x == 0 ? -y : y) < 0 ? 0 : SLEEF_INFINITYf);
+  if (xisinff(x) || x == 0) result = mulsignf((xsignbitf(y) ^ (x == 0)) ? 0 : SLEEF_INFINITYf, yisodd ? x : 1);
   if (xisnanf(x) || xisnanf(y)) result = SLEEF_NANf;
   if (y == 0 || x == 1) result = 1;
 
@@ -1918,56 +1914,56 @@ EXPORT CONST float xldexpf(float x, int exp) {
 }
 
 EXPORT CONST float xnextafterf(float x, float y) {
-  union {
-    float f;
-    int32_t i;
-  } cx;
+  float cxf;
+  int32_t cxi;
 
-  cx.f = x == 0 ? mulsignf(0, y) : x;
-  int c = (cx.i < 0) == (y < x);
-  if (c) cx.i = -(cx.i ^ (1 << 31));
+  cxf = x == 0 ? mulsignf(0, y) : x;
+  memcpy(&cxi, &cxf, sizeof(cxi));
+  int c = (cxi < 0) == (y < x);
+  if (c) cxi = -(cxi ^ (1 << 31));
 
-  if (x != y) cx.i--;
+  if (x != y) cxi--;
 
-  if (c) cx.i = -(cx.i ^ (1 << 31));
+  if (c) cxi = -(cxi ^ (1 << 31));
 
-  if (cx.f == 0 && x != 0) cx.f = mulsignf(0, x);
-  if (x == 0 && y == 0) cx.f = y;
-  if (xisnanf(x) || xisnanf(y)) cx.f = SLEEF_NANf;
+  memcpy(&cxf, &cxi, sizeof(cxf));
+  if (cxf == 0 && x != 0) cxf = mulsignf(0, x);
+  if (x == 0 && y == 0) cxf = y;
+  if (xisnanf(x) || xisnanf(y)) cxf = SLEEF_NANf;
   
-  return cx.f;
+  return cxf;
 }
 
 EXPORT CONST float xfrfrexpf(float x) {
-  union {
-    float f;
-    int32_t u;
-  } cx;
+  float cxf;
+  int32_t cxu;
 
   if (fabsfk(x) < FLT_MIN) x *= (1 << 30);
   
-  cx.f = x;
-  cx.u &= ~0x7f800000U;
-  cx.u |=  0x3f000000U;
+  cxf = x;
+  memcpy(&cxu, &cxf, sizeof(cxu));
 
-  if (xisinff(x)) cx.f = mulsignf(SLEEF_INFINITYf, x);
-  if (x == 0) cx.f = x;
+  cxu &= ~0x7f800000U;
+  cxu |=  0x3f000000U;
+
+  memcpy(&cxf, &cxu, sizeof(cxf));
+  if (xisinff(x)) cxf = mulsignf(SLEEF_INFINITYf, x);
+  if (x == 0) cxf = x;
   
-  return cx.f;
+  return cxf;
 }
 
 EXPORT CONST int xexpfrexpf(float x) {
-  union {
-    float f;
-    uint32_t u;
-  } cx;
+  float cxf;
+  uint32_t cxu;
 
   int ret = 0;
   
   if (fabsfk(x) < FLT_MIN) { x *= (1 << 30); ret = -30; }
   
-  cx.f = x;
-  ret += (int32_t)(((cx.u >> 23) & 0xff)) - 0x7e;
+  cxf = x;
+  memcpy(&cxu, &cxf, sizeof(cxu));
+  ret += (int32_t)(((cxu >> 23) & 0xff)) - 0x7e;
 
   if (x == 0 || xisnanf(x) || xisinff(x)) ret = 0;
   
@@ -2246,7 +2242,7 @@ typedef struct {
 } df2;
 
 static CONST df2 gammafk(float a) {
-  Sleef_float2 clc = df(0, 0), clln = df(1, 0), clld = df(1, 0), v = df(1, 0), x, y, z;
+  Sleef_float2 clc = df(0, 0), clln = df(1, 0), clld = df(1, 0), x, y, z;
   float t, u;
 
   int otiny = fabsfk(a) < 1e-30f, oref = a < 0.5f;
@@ -2318,32 +2314,61 @@ EXPORT CONST float xlgammaf_u1(float a) {
   return r;
 }
 
-EXPORT CONST float xerff_u1(float a) {
-  float s = a, t, u;
-  Sleef_float2 d;
+static INLINE CONST Sleef_float2 dfmla(float x, Sleef_float2 y, Sleef_float2 z) {
+  return dfadd2_f2_f2_f2(z, dfmul_f2_f2_f(y, x));
+}
+static INLINE CONST Sleef_float2 poly2df_b(float x, Sleef_float2 c1, Sleef_float2 c0) { return dfmla(x, c1, c0); }
+static INLINE CONST Sleef_float2 poly2df(float x, float c1, Sleef_float2 c0) { return dfmla(x, df(c1, 0), c0); }
+static INLINE CONST Sleef_float2 poly4df(float x, float c3, Sleef_float2 c2, Sleef_float2 c1, Sleef_float2 c0) {
+  return dfmla(x*x, poly2df(x, c3, c2), poly2df_b(x, c1, c0));
+}
 
-  a = fabsfk(a);
-  int o0 = a < 1.1f, o1 = a < 2.4f, o2 = a < 4.0f;
-  u = o0 ? (a*a) : a;
-  
-  t = o0 ? +0.7089292194e-4f : o1 ? -0.1792667899e-4f : -0.9495757695e-5f;
-  t = mlaf(t, u, o0 ? -0.7768311189e-3f : o1 ? +0.3937633010e-3f : +0.2481465926e-3f);
-  t = mlaf(t, u, o0 ? +0.5159463733e-2f : o1 ? -0.3949181177e-2f : -0.2918176819e-2f);
-  t = mlaf(t, u, o0 ? -0.2683781274e-1f : o1 ? +0.2445474640e-1f : +0.2059706673e-1f);
-  t = mlaf(t, u, o0 ? +0.1128318012e+0f : o1 ? -0.1070996150e+0f : -0.9901899844e-1f);
-  d = dfmul_f2_f_f(t, u);
-  d = dfadd2_f2_f2_f2(d, o0 ? dfx(-0.376125876000657465175213237214e+0) :
-		      o1 ? dfx(-0.634588905908410389971210809210e+0) :
-		      dfx(-0.643598050547891613081201721633e+0));
-  d = dfmul_f2_f2_f(d, u);
-  d = dfadd2_f2_f2_f2(d, o0 ? dfx(+0.112837916021059138255978217023e+1) :
-		      o1 ? dfx(-0.112879855826694507209862753992e+1) :
-		      dfx(-0.112461487742845562801052956293e+1));
-  d = dfmul_f2_f2_f(d, a);
-  d = o0 ? d : dfadd_f2_f_f2(1.0, dfneg_f2_f2(expk2f(d)));
-  u = mulsignf(o2 ? (d.x + d.y) : 1, s);
-  u = xisnanf(a) ? SLEEF_NANf : u;
-  return u;
+EXPORT CONST float xerff_u1(float a) {
+  float t, x = fabsfk(a);
+  Sleef_float2 t2;
+  float x2 = x * x, x4 = x2 * x2;
+
+  if (x < 2.5) {
+    // Abramowitz and Stegun
+    t = POLY6(x, x2, x4,
+	      -0.4360447008e-6,
+	      +0.6867515367e-5,
+	      -0.3045156700e-4,
+	      +0.9808536561e-4,
+	      +0.2395523916e-3,
+	      +0.1459901541e-3);
+    t2 = poly4df(x, t,
+		 df(0.0092883445322513580322, -2.7863745897025330755e-11),
+		 df(0.042275499552488327026, 1.3461399289988106057e-09),
+		 df(0.070523701608180999756, -3.6616309318707365163e-09));
+    t2 = dfadd_f2_f_f2(1, dfmul_f2_f2_f(t2, x));
+    t2 = dfsqu_f2_f2(t2);
+    t2 = dfsqu_f2_f2(t2);
+    t2 = dfsqu_f2_f2(t2);
+    t2 = dfsqu_f2_f2(t2);
+    t2 = dfrec_f2_f2(t2);
+  } else if (x > 4.0) {
+    t2 = df(0, 0);
+  } else {
+    t = POLY6(x, x2, x4,
+	      -0.1130012848e-6,
+	      +0.4115272986e-5,
+	      -0.6928304356e-4,
+	      +0.7172692567e-3,
+	      -0.5131045356e-2,
+	      +0.2708637156e-1);
+    t2 = poly4df(x, t,
+		 df(-0.11064319312572479248, 3.7050452777225283007e-09),
+		 df(-0.63192230463027954102, -2.0200432585073177859e-08),
+		 df(-1.1296638250350952148, 2.5515120196453259252e-08));
+    t2 = dfmul_f2_f2_f(t2, x);
+    t2 = df(expkf(t2), 0);
+  }
+
+  t2 = dfadd2_f2_f2_f(t2, -1);
+
+  if (x < 1e-4) t2 = dfmul_f2_f2_f(df(-1.1283792257308959961, 5.8635383422197591097e-08), x);
+  return mulsignf(a == 0 ? 0 : (xisinff(a) ? 1 : (-t2.x - t2.y)), a);
 }
 
 EXPORT CONST float xerfcf_u15(float a) {
